/*
 * This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/bonsai
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
 *              Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
 *              Marian Pohling, Lukas Twardon
 */
package de.unibi.airobots.bonsai.skills;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import org.apache.commons.scxml.SCXMLListener;
import org.apache.commons.scxml.model.Transition;
import org.apache.commons.scxml.model.TransitionTarget;

/**
 * Quick GUI to demonstrate a SCXML driven skill.
 * 
 * Separation of UI (this class) from behavior (? extends AbstractState classes)
 * . UI serves merely as a front to relay user initiated events to a Skill,
 * which encapsulates all the behavior of a skill. Using SCXML makes the skill
 * classes simplistic, and provides a direct route from the UML model to the
 * runtime.
 * 
 * @see SkillStateMachineImplementation
 */
public class StateGui extends JFrame implements ActionListener, SCXMLListener {

    private static final long serialVersionUID = 1L;

    private SkillStateMachine skillEngine;

    private JList currentState;
    private DefaultListModel currentStateListModel = new DefaultListModel();

    private JList nextStates;
    private DefaultListModel nextStateListModel = new DefaultListModel();
    
    private JButton sendEvent, start;
    private JComboBox eventList;
    JPanel currentStatePanel = new JPanel();

    /**
     * Constructor which actually needs the skill state machine implementation
     * used.
     * 
     * @param skillEngine
     */
    public StateGui(SkillStateMachine skillEngine) {
        super("SCXML skill execution Display");
        this.skillEngine = skillEngine;
        this.skillEngine.getEngine().addListener(
                this.skillEngine.getEngine().getStateMachine(), this);
        setupUI();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        String command = e.getActionCommand();
        if (command.equals("send")) {
            skillEngine.fireEvent(eventList.getSelectedItem().toString());
            nextStates.add(new JLabel("send Event via GUI: "
                    + eventList.getSelectedItem().toString() + "!\n"));

            eventList.removeAllItems();
            for (String aString : skillEngine.getEventList()) {
                eventList.addItem(aString);
            }
        } else if (command.equals("Start")) {
            start.setText("RESET");
            skillEngine.startMachine();
            for (String aString : skillEngine.getEventList()) {
                eventList.addItem(aString);
            }
        } else if (command.equals("RESET")) {
            start.setText("Start");
            skillEngine.resetMachine();
        }

    }

    /**
     * Simple method which sets up the GUI
     */
    private void setupUI() {

        
        currentStatePanel.setLayout(new FlowLayout());

        
        JPanel mainPanel = new JPanel();
        JPanel panel = new JPanel();
        mainPanel.setLayout(new BorderLayout());
        panel.setLayout(new FlowLayout());
        setContentPane(mainPanel);
        currentState = new JList(currentStateListModel);
        currentState.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        currentState.setLayoutOrientation(JList.VERTICAL);
        currentState.setVisibleRowCount(20);
        currentState.setPreferredSize(new Dimension(500, 300));

        JScrollPane currentStateScrollingPane = new JScrollPane(currentState);
        currentStateScrollingPane
                .setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

        nextStates = new JList(nextStateListModel);
        nextStates.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        nextStates.setLayoutOrientation(JList.VERTICAL);
        nextStates.setVisibleRowCount(20);
        // nextStates.setRows(15);
        nextStates.setPreferredSize(new Dimension(500, 300));
        JScrollPane nextStatesScrollingPane = new JScrollPane(nextStates);
        nextStatesScrollingPane
                .setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

        panel.add(currentStateScrollingPane);
        panel.add(nextStatesScrollingPane);
        
        mainPanel.add(panel, BorderLayout.CENTER);

        JPanel panel2 = new JPanel();
        panel2.setLayout(new FlowLayout());
        mainPanel.add(panel2, BorderLayout.SOUTH);

        start = makeButton("Start", "Start", "Start");
        panel2.add(start);

        // eventList = new JComboBox(skillEngine.getEventList().toArray());
        eventList = new JComboBox();
        panel2.add(eventList);

        sendEvent = makeButton("send", "send an event...", "send");
        panel2.add(sendEvent);

        pack();
        setLocation(300, 100);

        // setIconImage(watchIcon);

        // setSize(300, 400);
        show();

        setDefaultCloseOperation(EXIT_ON_CLOSE);
    }

    private JButton makeButton(final String actionCommand,
            final String toolTipText, final String altText) {
        JButton button = new JButton(altText);
        button.setActionCommand(actionCommand);
        button.setToolTipText(toolTipText);
        button.addActionListener(this);
        button.setOpaque(false);
        return button;
    }

    @Override
    public void onEntry(TransitionTarget arg0) {
        // arg0.get

    }

    @Override
    public void onExit(TransitionTarget arg0) {
        // System.out.println("ASDASF EEEXIT");

    }

    @Override
    public void onTransition(TransitionTarget arg0, TransitionTarget arg1,
            Transition arg2) {
        currentStateListModel.addElement(("Current state ID: " + arg0.getId() + "\n"));


        // for (int i = 0; i < arg1.getTransitionsList().size(); i++) {
        // nextStates.append(arg1.getTransitionsList().get(i).toString() +
        // "\n");
        // }
        nextStateListModel.addElement(("Event triggered: " + arg2.getEvent()));

        this.repaint();
        eventList.removeAllItems();
        for (String aString : skillEngine.getEventList()) {
            eventList.addItem(aString);
        }
    }

}
