/*
 * This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
 *
 * Copyright(c) Frederic Siepmann
 * http://opensource.cit-ec.de/projects/bonsai
 *
 * This file may be licensed under the terms of of the
 * GNU Lesser General Public License Version 3 (the ``LGPL''),
 * or (at your option) any later version.
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the LGPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the LGPL along with this
 * program. If not, go to http://www.gnu.org/licenses/lgpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * The development of this software was supported by the
 * Excellence Cluster EXC 277 Cognitive Interaction Technology.
 * The Excellence Cluster EXC 277 is a grant of the Deutsche
 * Forschungsgemeinschaft (DFG) in the context of the German
 * Excellence Initiative.
 *
 * Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
 * 				Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
 * 				Marian Pohling, Lukas Twardon
 */
package de.unibi.airobots.bonsai.skills.state;

import java.io.IOException;

import org.apache.log4j.Logger;

import de.unibi.airobots.bonsai.actuators.EnterArenaAndLocalizeActuator;
import de.unibi.airobots.bonsai.actuators.NavigationActuator;
import de.unibi.airobots.bonsai.actuators.PoseActuatorTobi;
import de.unibi.airobots.bonsai.actuators.SpeechActuator;
import de.unibi.airobots.bonsai.core.Sensor;
import de.unibi.airobots.bonsai.skills.SkillStateMachine;
import de.unibi.airobots.bonsai.util.helper.Movements;
import de.unibi.airobots.btl.data.GlobalPlannerStateChange;
import de.unibi.airobots.btl.data.LaserData;
import de.unibi.airobots.btl.data.NavigationGoalData;
import de.unibi.airobots.btl.data.NavigationGoalData.GoalType;
import de.unibi.airobots.btl.data.PositionData;
import de.unibi.airobots.btl.data.SpeedData;

/**
 * Abstract definition of a state.
 * 
 * @author nkoester
 */
public abstract class AbstractState extends Thread{

    /**
     * 
     */
    SkillStateMachine stateMachine;

    private Sensor<GlobalPlannerStateChange> gplanChangeSensor;
    private Sensor<LaserData> laserSensor;
    private Sensor<PositionData> positionSensor;
    private Sensor<SpeedData> speedSensor;
    private EnterArenaAndLocalizeActuator enterArenaActuator;
    private NavigationActuator navActuator;
    private PoseActuatorTobi poseActuatorTobi;
    private SpeechActuator speechActuator;
    
    
    
    
    
   

    /**
     * @param aStateMachine
     */
    public AbstractState(SkillStateMachine aStateMachine) {
        System.out.println(this.getClass().getSimpleName()
                + " Constructor called");
        this.stateMachine = aStateMachine;
        this.getRequiredSensorsAndActuators();
    }

    /**
     * {@inheritDoc}
     */
    public void run(){
        try {
            if (init()) {
                System.out.println(this.getClass() + " finished init!");
                if (execute()) {
                    System.out.println(this.getClass() + " finished execute!");
                    if (end()) {
                        System.out.println(this.getClass() + " finished end!");
                        stateMachine.fireEvent(this.getClass().getSimpleName()
                                    + ".successful");
                    } else {
                        System.out.println("\t" + this.getClass() + " end FAILED!");
                        stateMachine.fireEvent(this.getClass().getSimpleName()
                                + ".fail");
                    }
                } else {
                    System.out.println("\t" + this.getClass() + " exec FAILED!");
                    stateMachine.fireEvent(this.getClass().getSimpleName()
                            + ".fail");
                }
            } else {
                System.out.println("\t" + this.getClass() + " init FAILED!");
                stateMachine.fireEvent(this.getClass().getSimpleName()
                        + ".fail");
            }
        } catch (Exception e) {
            System.out.println("\t" + this.getClass() + " run FAIL!");
            e.printStackTrace();
            stateMachine.fireEvent(this.getClass().getSimpleName() + ".fail");
        }
    }
    
    
    /**
     * The logger.
     */
    protected Logger logger = Logger.getLogger(this.getClass());

    /**
     * Defines whether a state is interruptible at the moment or not.
     */
    private boolean interruptible = true;

    /**
     * Sets up the initial references and parameters required by a state.
     * 
     * @return true on success
     */
    public abstract boolean init();

    /**
     * Main function of a state that contains the actual logic and (sub-) skill.
     * 
     * @return true on success
     */
    public abstract boolean execute();

    /**
     * Function that is called after the main functional part of a state has
     * been executed.
     * 
     * @return true on success
     */
    public abstract boolean end();

    /**
     * Sets the interruptible status of a state.
     * 
     * @param newInterruptStatus
     *            the new status
     */
    private void setInterruptible(boolean newInterruptStatus) {
        this.interruptible = newInterruptStatus;
    }

    /**
     * Checks whether a state is interruptible at the moment or not.
     * 
     * @return true if the current state is interruptible
     */
    public boolean isInterruptible() {
        return interruptible;
    }
    
    public void requestInterruptible() {
        //TODO: implement
    }
    
    public abstract void getRequiredSensorsAndActuators();
    
   
}
