/*
* This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/bonsai
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
* 				Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
* 				Marian Pohling, Lukas Twardon
*/
package de.unibi.airobots.bonsai.strategies.exploration;

import java.util.Vector;

import org.apache.log4j.Logger;

import de.unibi.airobots.bonsai.util.helper.annotation.MapRegion;
import de.unibi.airobots.btl.data.NavigationGoalData;
import de.unibi.airobots.btl.data.PositionData;
import de.unibi.airobots.btl.data.NavigationGoalData.GoalType;

public class ViewPointBasedSearchStrategy implements ExplorationStrategy {

    private Logger logger = Logger.getLogger(this.getClass());

    static final double COORDINATE_TOLERANCE = 0.5; // meters

    Vector<PositionData> allViewPoints = new Vector<PositionData>();
    Vector<PositionData> remainingViewPoints = new Vector<PositionData>();
    Vector<PositionData> missmatch = new Vector<PositionData>();

    void addViewPoint(PositionData point) {
        logger.debug("addPoint " + point);
        allViewPoints.add(point);
        remainingViewPoints.add(point);
    }

    void addMissMatchPoint(PositionData point) {
        missmatch.add(point);
    }

    void removeViewPoint(PositionData point) {
        allViewPoints.remove(point);
        remainingViewPoints.remove(point);
    }

    @Override
    public NavigationGoalData getNextGoal(PositionData currentPosition) {

        logger.info("getNextGoal");

        if (remainingViewPoints.isEmpty()) {
            if (missmatch.isEmpty())
                remainingViewPoints.addAll(allViewPoints);
            else
                remainingViewPoints.addAll(missmatch);
        }

        logger.debug(remainingViewPoints);

        logger.debug("currentPosition " + currentPosition);

        double minDist = Double.MAX_VALUE;
        PositionData minDistPos = null;
        for (PositionData p : remainingViewPoints) {
            logger.debug("PositionData " + p);
            double d =
                    Math.sqrt(Math.pow(p.getX() - currentPosition.getX(), 2)
                            + Math.pow(p.getY() - currentPosition.getY(), 2));
            logger.debug("minDist " + minDist + " d " + d);
            if (d < minDist) {
                minDist = d;
                minDistPos = p;
            }
        }
        remainingViewPoints.remove(minDistPos);
        logger.debug("goarl " + minDistPos);
        NavigationGoalData goal = new NavigationGoalData(minDistPos);
        goal.setCoordinateTolerance(COORDINATE_TOLERANCE);
        goal.setType(GoalType.GLOBAL);
        return goal;
    }

    @Override
    public NavigationGoalData getNextGoal(PositionData currentPosition,
            MapRegion roomPosition) {
        logger.info("getNextGoal");

        if (remainingViewPoints.isEmpty()) {
            if (missmatch.isEmpty())
                remainingViewPoints.addAll(allViewPoints);
            else
                remainingViewPoints.addAll(missmatch);
        }

        logger.debug(remainingViewPoints);

        logger.debug("currentPosition " + currentPosition);

        double minDist = Double.MAX_VALUE;
        PositionData minDistPos = null;
        for (PositionData p : remainingViewPoints) {
            logger.debug("PositionData " + p);
            logger.debug("roomPosition " + roomPosition.getShape());
            double d =
                    Math.sqrt(Math.pow(p.getX() - currentPosition.getX(), 2)
                            + Math.pow(p.getY() - currentPosition.getY(), 2));
            logger.debug("minDist " + minDist + " d " + d);
            if (d < minDist && roomPosition.contains(p.getX(), p.getY())) {
                minDist = d;
                minDistPos = p;
            }
        }
        remainingViewPoints.remove(minDistPos);
        logger.debug("goarl " + minDistPos);
        if (minDistPos == null)
            minDistPos = roomPosition.getViewPoint();
        NavigationGoalData goal = new NavigationGoalData(minDistPos);
        goal.setCoordinateTolerance(COORDINATE_TOLERANCE);
        goal.setType(GoalType.GLOBAL);
        return goal;
    }

}
