/*
* This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/bonsai
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
* 				Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
* 				Marian Pohling, Lukas Twardon
*/
package de.unibi.airobots.bonsai.xcf.actuators;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import net.sf.xcf.MethodNotFoundException;
import net.sf.xcf.RemoteServer;
import net.sf.xcf.UserException;
import net.sf.xcf.transport.XOPData;

import org.apache.log4j.Logger;

import de.unibi.airobots.bonsai.Globals;
import de.unibi.airobots.bonsai.actuators.StateVisualizerActuator;
import de.unibi.airobots.bonsai.xcf.RemoteServerReconnectable;
import de.unibi.airobots.btl.data.StateChangeRequest;
import de.unibi.airobots.btl.data.StateConfiguration;

/**
 * Actuator that controls the state visualizer. The state visualizer can be used
 * to give a short graphical feedback on which state the system is currently in.
 * 
 * @author jwienke
 */
public class XcfStateVisualizerActuator implements StateVisualizerActuator,
        RemoteServerReconnectable {

    private Logger logger = Logger.getLogger(getClass());

    private String remoteServerName = "";
    private Object remoteServerLock = new Object();
    private RemoteServer remoteServer = null;

    /**
     * Constructor.
     * 
     * @param remoteServerName
     *            the name of the remote server to use
     */
    public XcfStateVisualizerActuator(String remoteServerName) {
        this.remoteServerName = remoteServerName;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void configure(String systemName, Set<String> states)
            throws IOException {

        synchronized (remoteServerLock) {
            if (this.remoteServer == null) {
                throw new IllegalStateException("No remote server set");
            }
        }

        StateConfiguration config = new StateConfiguration();
        config.setGenerator(Globals.LIBRARY_NAME);
        config.setSystemName(systemName);
        config.addStates(states);

        logger.debug("Configuring with: " + config);

        try {
            synchronized (remoteServerLock) {
                remoteServer.callMethod("configure", new XOPData(config
                        .toDocument(StateConfiguration.class)));
            }
        } catch (MethodNotFoundException e) {
            throw new IOException(e);
        } catch (UserException e) {
            throw new IOException(e);
        }

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void setState(String state) throws IOException {

        synchronized (remoteServerLock) {
            if (this.remoteServer == null) {
                throw new IllegalStateException("No remote server set");
            }
        }

        StateChangeRequest request = new StateChangeRequest();
        request.setGenerator(Globals.LIBRARY_NAME);
        request.setNewStateName(state);

        logger.debug("Setting state with: " + request);

        try {
            synchronized (remoteServerLock) {
                remoteServer.callMethod("setState", new XOPData(request
                        .toDocument(StateChangeRequest.class)));
            }
        } catch (MethodNotFoundException e) {
            throw new IOException(e);
        } catch (UserException e) {
            throw new IOException(e);
        }

    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Map<String, ErrorHandlingPolicy> getRemoteServerNames() {
        Map<String, ErrorHandlingPolicy> servers =
                new HashMap<String, ErrorHandlingPolicy>();
        servers.put(remoteServerName, ErrorHandlingPolicy.SELF_HEALING);
        return servers;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void resetRemoteServer(RemoteServer remoteServer) {
        assert remoteServer.getServerName().equals(remoteServerName);
        synchronized (remoteServerLock) {
            this.remoteServer = remoteServer;
        }
    }

}
