/*
* This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/bonsai
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
* 				Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
* 				Marian Pohling, Lukas Twardon
*/
package de.unibi.airobots.bonsai.xcf.sensors;

import java.io.IOException;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import net.sf.xcf.Subscriber;
import net.sf.xcf.event.PublishEvent;
import net.sf.xcf.event.PublishEventAdapter;
import net.sf.xcf.event.PublishEventListener;
import net.sf.xcf.event.XcfEvent;
import net.sf.xcf.transport.Attachment;

import de.unibi.airobots.bonsai.core.Sensor;
import de.unibi.airobots.bonsai.core.SensorListener;
import de.unibi.airobots.bonsai.sensors.data.ImageData;
import de.unibi.airobots.bonsai.sensors.data.PlaneType;
import de.unibi.airobots.bonsai.xcf.BoundSynchronizedQueue;
import de.unibi.airobots.bonsai.xcf.SubscriberReconnectable;

/**
 * The CameraSensor receives the current ImageData provided by the robot.
 * 
 * @author sebschne
 * @author jwienke
 */
public class XcfCameraSensor implements Sensor<ImageData>,
        SubscriberReconnectable {

    private static final int PLANE_NUMBER_GRAY = 1;

    private static final int PLANE_NUMBER_COLOR = 3;

    private BoundSynchronizedQueue<PublishEvent> queue =
            new BoundSynchronizedQueue<PublishEvent>() {

                @Override
                public PublishEvent convertFromXcfEvent(XcfEvent e) {
                    // not used here
                    return null;
                }
            };

    private String subscriberName = "";
    private Subscriber subscriber = null;

    private static final String CS_RGB = "rgb";
    private static final String CS_YUV = "yuv";
    private static final String CS_GRAY = "gray";

    private Set<SensorListener<ImageData>> listeners =
            new CopyOnWriteArraySet<SensorListener<ImageData>>();

    private PublishEventListener listenerAdapter = new ListenerNotifier();

    /**
     * An adapter that maps a {@link PublishEvent} to all registered
     * {@link SensorListener}s.
     * 
     * @author jwienke
     */
    private class ListenerNotifier extends PublishEventAdapter {

        /**
         * {@inheritDoc}
         */
        @Override
        public void handleEvent(PublishEvent event) {

            queue.push(event);

            if (!listeners.isEmpty()) {

                ImageData image = fromPublishEvent(event);
                for (SensorListener<ImageData> l : listeners) {
                    l.newDataAvailable(image);
                }

            }

        }

    }

    /**
     * Convert a {@link PublishEvent} to {@link ImageData}.
     * 
     * @param event
     *            event to convert
     * @return converted {@link ImageData}
     * @throws IllegalArgumentException
     *             converting failed
     */
    private ImageData fromPublishEvent(PublishEvent event)
            throws IllegalArgumentException {

        try {

            de.unibi.airobots.btl.data.ImageData btlData =
                    new de.unibi.airobots.btl.data.ImageData();
            de.unibi.airobots.btl.data.ImageData.fromElement(event.getData()
                    .getDocument().getRootElement(), btlData);

            int nplanes = PLANE_NUMBER_COLOR;
            PlaneType ptype = PlaneType.UNKNOWN_RAW8;
            if (CS_RGB.equals(btlData.getColorspace())) {
                ptype = PlaneType.PLANAR_RGB8;
            } else if (CS_YUV.equals(btlData.getColorspace())) {
                ptype = PlaneType.PLANAR_YUV8;
            } else if (CS_GRAY.equals(btlData.getColorspace())) {
                ptype = PlaneType.GRAY8;
                nplanes = PLANE_NUMBER_GRAY;
            }
            if (!(nplanes == PLANE_NUMBER_GRAY || nplanes == PLANE_NUMBER_COLOR)) {
                throw new IllegalArgumentException(
                        "Expected 1 or 3 planes, not " + nplanes);
            }

            // get attachment
            Attachment imagePacket =
                    event.getData().getAttachment(btlData.getUri());

            final int expectedSize =
                    btlData.getWidth() * btlData.getHeight() * nplanes;
            if (imagePacket.getValue().length == expectedSize) {
                return new ImageData(btlData.getWidth(), btlData.getHeight(),
                        btlData.getUri(), imagePacket.getValue(), 0, ptype);
            } else {
                // old-school packet with meta-data header
                return new ImageData(btlData.getWidth(), btlData.getHeight(),
                        btlData.getUri(), imagePacket.getValue(), imagePacket
                                .getValue().length
                                - expectedSize, ptype);
            }

            // CHECKSTYLE:OFF catch everything here as error
        } catch (Exception e) {
            // CHECKSTYLE:ON
            throw new IllegalArgumentException(
                    "Error parsing image from xcf event", e);
        }

    }

    /**
     * The CameraSensor needs a Subscriber who streams Images over the
     * XCFImageExport stream.
     * 
     * @param subscriberName
     *            name of the subscriber to work on
     */
    public XcfCameraSensor(String subscriberName) {
        this.subscriberName = subscriberName;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void addSensorListener(SensorListener<ImageData> listener) {
        listeners.add(listener);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Class<ImageData> getDataType() {
        return ImageData.class;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean hasNext() {
        return this.queue.hasNext();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public ImageData readLast() throws IOException, InterruptedException {
        try {
            return fromPublishEvent(queue.next());
        } catch (IllegalArgumentException e) {
            throw new IOException(e);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public ImageData readLast(long timeout) throws IOException,
            InterruptedException {
        try {
            PublishEvent event = queue.next(timeout);
            if (event != null) {
                return fromPublishEvent(event);
            }
            return null;
        } catch (IllegalArgumentException e) {
            throw new IOException(e);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void removeSensorListener(SensorListener<ImageData> listener) {
        listeners.remove(listeners);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getSubscriberName() {
        return subscriberName;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void resetSubscriber(Subscriber subscriber) {
        if (this.subscriber != null) {
            this.subscriber.removeListener(listenerAdapter);
        }
        this.subscriber = subscriber;
        this.subscriber.addListener(listenerAdapter);
    }

}
