package de.unibi.airobots.bonsai.xcf.sensors;

import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import net.sf.xcf.ActiveMemory;
import net.sf.xcf.event.MemoryEvent;
import net.sf.xcf.event.MemoryEventAdapter;
import net.sf.xcf.event.MemoryEventListener;
import net.sf.xcf.event.XcfEvent;
import net.sf.xcf.memory.MemoryAction;
import net.sf.xcf.memory.MemoryException;
import net.sf.xcf.xml.XPath;
import nu.xom.Attribute;
import nu.xom.Element;

import org.apache.log4j.Logger;

import de.unibi.airobots.bonsai.core.SensorListener;
import de.unibi.airobots.bonsai.sensors.MapAnnotationSensor;
import de.unibi.airobots.bonsai.xcf.ActiveMemoryReconnectable;
import de.unibi.airobots.bonsai.xcf.BoundSynchronizedQueue;
import de.unibi.airobots.btl.data.annotation.MapAnnotation;

/**
 * This is a Sensor that listens to a MapAnnotation on the Memory.
 * 
 * @author vrichter
 */
public class XcfMapAnnotationSensor implements MapAnnotationSensor,
		ActiveMemoryReconnectable {

	private Logger logger = Logger.getLogger(getClass());

	private String memoryName = "";
	private Object memoryLock = new Object();
	private Object listenersLock = new Object();
	private ActiveMemory memory = null;

	/**
	 * Namespace of active memory xml documents.
	 */
	public static final String NAMESPACE_DBXML = "http://www.sleepycat.com/2002/dbxml";

	private List<SensorListener<MapAnnotation>> listeners = new LinkedList<SensorListener<MapAnnotation>>();

	private BoundSynchronizedQueue<MemoryEvent> queue = new BoundSynchronizedQueue<MemoryEvent>() {
		@Override
		public MemoryEvent convertFromXcfEvent(XcfEvent e) {
			// not used here
			return null;
		}
	};

	/**
	 * The MapAnnotation is saved here when it is parsed. New MemoryEvents set
	 * this field to null.
	 */
	private MapAnnotation map;

	/**
	 * Own listener used to intercept the memory for annotation changes.
	 */
	private MemoryEventListener selfListener;

	/**
	 * Constructor.
	 * 
	 * @param memoryName
	 *            name of the active memory to get the MapAnnotation from
	 */
	public XcfMapAnnotationSensor(String memoryName) {

		this.memoryName = memoryName;

		logger.debug("Create memory sensor for memoryName " + memoryName);

		selfListener = new MemoryEventAdapter(MemoryAction.ALL, new XPath(
				MapAnnotation.getBaseTag())) {

			public void handleEvent(MemoryEvent e) {
				logger.debug("inside handleMemoryEvent");
				handleMemoryEvent(e);
			}
		};
		// memory.addListener(selfListener);
	}

	protected void handleMemoryEvent(MemoryEvent e) {
		try {
			logger.debug("inside handleMemoryEvent");
			MemoryAction eventType = e.getType();
			logger.debug("MemoryAction " + eventType);

			if (eventType == MemoryAction.QUERY) {
				logger.debug("query, ignoring");
				return;
			}

			if (eventType == MemoryAction.REMOVE) {
				logger.debug("remove");
				queue.clear();
			}

			Element root = e.getData().getDocument().getRootElement();
			if (root == null) {
				logger.error("Received Document has no root node");
				return;
			}

			// extract xcf id
			Attribute att = root.getAttribute("id", NAMESPACE_DBXML);
			if (att == null) {
				logger.error("Received Document has no xcf id");
				return;
			}
			int id = Integer.parseInt(att.getValue());

			if (eventType == MemoryAction.INSERT
					|| eventType == MemoryAction.REPLACE) {
				logger.debug("insert id " + id);
				queue.push(e);
				map = null;
			}
			logger.debug("Listeners count: " + listeners.size());

			if (listeners.size() > 0) {
				MapAnnotation map = createMapAnnotation(e);
				for (SensorListener<MapAnnotation> l : listeners) {
					l.newDataAvailable(map);
				}
			}
		} catch (NumberFormatException ex) {
			logger.error("Received Document has not a valid xcf id");
			logger.debug(ex);
		}

	}

	/**
	 * Creates a MapAnnotation from a MemoryEvent calling the fromElement method
	 * of MapAnnotation.
	 * 
	 * @param e
	 *            A MemoryEvent containing a MapAnnotation as root element
	 * @return The corresponding MapAnnotation
	 */
	private MapAnnotation createMapAnnotation(MemoryEvent e) {
		if (this.map == null) {
			Element root = e.getData().getDocument().getRootElement();
			// create BTL type
			map = new MapAnnotation();
			MapAnnotation.fromElement(root, map);
		}
		return this.map;
	}

	@Override
	public void addSensorListener(SensorListener<MapAnnotation> listener) {
		synchronized (listenersLock) {
			listeners.add(listener);
		}
	}

	@Override
	public void removeSensorListener(SensorListener<MapAnnotation> listener) {
		synchronized (listenersLock) {
			listeners.remove(listener);
		}
	}

	@Override
	public Class<MapAnnotation> getDataType() {
		return MapAnnotation.class;
	}

	@Override
	public boolean hasNext() {
		return this.queue.hasNext();
	}

	@Override
	public MapAnnotation readLast() throws IOException, InterruptedException {
		return createMapAnnotation(queue.next());
	}

	@Override
	public MapAnnotation readLast(long timeout) throws IOException,
			InterruptedException {
		MemoryEvent event = queue.next(timeout);
		if (event != null) {
			return createMapAnnotation(event);
		} else {
			return null;
		}
	}

	@Override
	public String getActiveMemoryName() {
		return this.memoryName;
	}

	@Override
	public void resetActiveMemory(ActiveMemory activeMemory) {
		synchronized (memoryLock) {
			this.memory = activeMemory;
			try {
				memory.addListener(selfListener);
			} catch (MemoryException e) {
				logger.info("Could not add listener to memory.");
			}
		}
	}
}
